@@ -5,7 +5,7 @@ The implementation of the fusion algorithm
 """
 from collections import Counter
 from itertools import chain
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING, Iterator, Optional
 
 from tilings.assumptions import (
     ComponentAssumption,
@@ -69,7 +69,7 @@ class Fusion:
         ], "The only valid isolation levels are None, 'noninteracting', and 'isolated'."
         self._fused_tiling: Optional["Tiling"] = None
 
-    def _fuse_gridded_perm(self, gp):
+    def fuse_gridded_perm(self, gp: GriddedPerm) -> GriddedPerm:
         """
         Fuse the gridded permutation `gp`.
         """
@@ -82,9 +82,14 @@ class Fusion:
             fused_pos.append((x, y))
         return gp.__class__(gp.patt, fused_pos)
 
-    def _unfuse_gridded_perm(self, gp):
+    def unfuse_gridded_perm(
+        self, gp: GriddedPerm, left_points: Optional[int] = None
+    ) -> Iterator[GriddedPerm]:
         """
         Generator of all the possible ways to unfuse a gridded permutations.
+
+        If left_points is given, the iterator contains only one gridded
+        permutations with said number of left points.
         """
 
         def stretch_above(p):
@@ -105,13 +110,20 @@ class Fusion:
                 i for i, p in enumerate(gp.pos) if p[0] == self._col_idx
             ]
             editable_pos_idx.sort()
+
         pos = list(map(stretch, gp.pos))
-        yield gp.__class__(gp.patt, pos)
+        if left_points is None or left_points == 0:
+            yield gp.__class__(gp.patt, pos)
+        if left_points == 0:
+            return
         row_shift = int(self._fuse_row)
         col_shift = 1 - int(self._fuse_row)
-        for i in editable_pos_idx:
+        for left_points_so_far, i in enumerate(editable_pos_idx):
             pos[i] = (pos[i][0] - col_shift, pos[i][1] - row_shift)
-            yield gp.__class__(gp.patt, pos)
+            if left_points is None or left_points_so_far + 1 == left_points:
+                yield gp.__class__(gp.patt, pos)
+            if left_points_so_far + 1 == left_points:
+                break
 
     def _fuse_counter(self, gridded_perms):
         """
@@ -122,7 +134,7 @@ class Fusion:
         """
         fuse_counter = Counter()
         for gp in gridded_perms:
-            fused_perm = self._fuse_gridded_perm(gp)
+            fused_perm = self.fuse_gridded_perm(gp)
             fuse_counter[fused_perm] += 1
         return fuse_counter
 
@@ -539,7 +551,7 @@ class ComponentFusion(Fusion):
         obstructions between first cell and second cell are not processed.
         """
         return chain.from_iterable(
-            self._unfuse_gridded_perm(ob) for ob in self.obstruction_fuse_counter
+            self.unfuse_gridded_perm(ob) for ob in self.obstruction_fuse_counter
         )
 
     def _can_fuse_assumption(self, assumption, fuse_counter):
