@@ -72,9 +72,9 @@ class RowColumnSeparationStrategy(DisjointUnionStrategy[Tiling, GriddedPerm]):
         )
         return (
             {
-                comb_class.get_parameter(assumption): child.get_parameter(
-                    mapped_assumption
-                )
+                comb_class.get_assumption_parameter(
+                    assumption
+                ): child.get_assumption_parameter(mapped_assumption)
                 for assumption, mapped_assumption in zip(
                     comb_class.assumptions, mapped_assumptions
                 )
